
namespace Midori
{
	errordomain FeatureError
	{
		NOT_SUPPORTED
	}

	public class App : Object
	{
		static App? instance = null;

		public static App get_default ()
		{
			if (instance == null) {
				instance = new App ();
			}
				
			return instance;
		}

		HashTable<string,Object> features;

		public App ()
		{
			features = new HashTable<string,Object> (str_hash, str_equal);
		}

		public void activate ()
		{
			register_feature ("search-engines", new SearchEngineManager ());
			register_feature ("history", new HistoryDatabase ());
			// register_feature ("bookmarks", new BookmarksDatabase ());

			var w = new Midori.Window ();
			w.set_default_size (1200, 800);
			w.show_all ();
			w.destroy.connect (Gtk.main_quit);
		}

		public Object get_feature (string id) throws Error
		{
			var feature = features[id];

			if (feature != null)
				return feature;

			throw new FeatureError.NOT_SUPPORTED ("Feature %s is not available".printf (id));
		}

		public void register_feature (string id, Object feature)
		{
			features[id] = feature;
		}
	}
}

void main (string[] args)
{
	Gtk.init (ref args);

	Midori.Paths.init_exec_path (args);
	Midori.Paths.init (Midori.RuntimeMode.NORMAL, null);

	var app = Midori.App.get_default ();
	app.activate ();

	Gtk.main ();
}

