/*
 Copyright (C) 2012-2013 Christian Dywan <christian@twotoasts.de>

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 See the file COPYING for the full license text.
*/

namespace Midori {
    public class SearchCompletion : Completion {

        SearchEngineManager? search_engine_manager = null;
        Icon default_icon;

        public SearchCompletion () {
            GLib.Object (description: _("Search with…"));

            default_icon = new ThemedIcon.with_default_fallbacks ("edit-find-symbolic");

            try {
                search_engine_manager = App.get_default ()
                    .get_feature ("search-engines") as SearchEngineManager;
            } catch (Error e) {
                search_engine_manager = null;
            }
        }

        public override void prepare () {
        }

        public override bool can_complete (string text) {
            return search_engine_manager != null;
        }

        public override bool can_action (string action) {
            return action == "complete:more/search";
        }

        public override async List<Suggestion>? complete (string text, string? action, Cancellable cancellable) {
            if (search_engine_manager == null)
                return null;

            var suggestions = new List<Suggestion> ();
            uint n = 0;
            foreach (var item in search_engine_manager.search_engines) {
                string search_uri = URI.for_search (item.uri, text);
                string search_title = _("Search with %s").printf (item.name);
                Icon? icon = yield Midori.URI.get_icon_fallback (item.uri, default_icon, cancellable);
                string search_desc = search_title + "\n" + item.text ?? item.uri;
                /* FIXME: Theming? Win32? */
                string background = "gray";
                var suggestion = new Suggestion (search_uri, search_desc, false, background, icon);
                suggestions.append (suggestion);

                n++;
                /*if (n == 3 && action == null) {
                    suggestion = new Suggestion ("complete:more/search", _("Search with…"), false, background);
                    suggestion.action = true;
                    suggestion.priority = this.position;
                    suggestions.append (suggestion);
                    break;
                }*/

                uint src = Idle.add (complete.callback);
                yield;
                Source.remove (src);
            }

            if (cancellable.is_cancelled ())
                return null;

            return suggestions;
        }
    }
}
