
namespace Midori
{
	public struct SearchEngine
	{
		string name;
		string text;
		string uri;
		string token;
	}

	public class SearchEngineManager : Object
	{
		public unowned List<SearchEngine?>? search_engines { get; private set; default = null; }

		public SearchEngineManager ()
		{
			load_from_folder ();
		}

		public string? query_matches_token (string uri)
		{
			if (search_engines == null)
				return null;

			foreach (var engine in search_engines) {
				if (uri.has_prefix (engine.token + " "))
					return engine.uri;
			}

			return null;
		}

		void load_from_file (string filename) throws Error
		{
			search_engines = new List<SearchEngine?> ();

			var key_file = new KeyFile ();
			key_file.load_from_file (filename, KeyFileFlags.KEEP_COMMENTS);

			foreach (var engine in key_file.get_groups ()) {
				search_engines.prepend ({
					key_file.get_string (engine, "name"),
					key_file.get_string (engine, "text"),
					key_file.get_string (engine, "uri"),
					key_file.get_string (engine, "token")
				});
			}
			search_engines.reverse ();
		}

		void load_from_folder ()
		{
			Error? error = null;

			try {
				load_from_file (Paths.get_config_filename_for_reading ("search"));
			} catch (Error e) {
				search_engines = null;
				error = e;
			}

			if (search_engines == null || search_engines.length () < 1) {
				try {
					load_from_file (Paths.get_preset_filename (null, "search"));
				} catch (Error e) {
					search_engines = null;
					error = e;
				}
			}

			if (search_engines == null) {
				warning (_("The search engines couldn't be loaded. %s\n"), error != null ? error.message : "");
				search_engines = new List<SearchEngine?> ();
			}
		}

		bool save_to_file (List<SearchEngine?> search_engines, string filename) throws Error
		{
			var key_file = new KeyFile ();

			foreach (var engine in search_engines) {
				key_file.set_string (engine.name, "name", engine.name);
				key_file.set_string (engine.name, "text", engine.text);
				key_file.set_string (engine.name, "uri", engine.uri);
				key_file.set_string (engine.name, "token", engine.token);
			}

			return key_file.save_to_file (filename);
		}
	}
}

