
public interface TabContent : Gtk.Widget
{
	public abstract string page_title { get; protected set; }
	public abstract Gdk.Pixbuf? icon { get; protected set; }
	public abstract string? ui_color { get; protected set; }
}

public interface Searchable : Object
{
	/**
	 * Start a full text search and highlight matches if requested
	 *
	 * @param text				The text to search for
	 * @param case_sensitive	Whether the search should be case sensitive
	 * @param highlight			Whether to highlight all matches
	 */
	public abstract void search (string text, bool case_sensitive, bool highlight);

	/**
	 * Jump to the next result
	 */
	public abstract void next_result ();

	/**
	 * Jump to the previous result
	 */
	public abstract void previous_result ();
}

public interface Navigatable : Object
{
	/**
	 * Add an entry to the history identified by a custom_id. @link{open_history_item}
	 * will be called with that id to notify the Navigatable that the user requested
	 * to move back to that item.
	 *
	 * @param custom_id The id of the history item
	 * @param title 	Optional title that may be shown to the user
	 */
	public signal void add_history_entry (string custom_id, string? title);

	/**
	 * Open the requested item given by the id which is the same as previously
	 * passed to @link{add_history_entry}
	 *
	 * @param id The id of the history item
	 */
	public abstract void open_history_item (string id);

	/**
	 * Request to open a new window at uri
	 *
	 * @param the uri to display in the new window
	 */
	public signal void request_open_new_window (string uri);
}

public interface Notifier : Object
{
	public enum DialogType
	{
		ALERT,
		PROMPT,
		CONFIRM
	}

	/**
	 * TODO
	 */
	public signal void open_dialog (DialogType type, string message);
}

public interface Savable : Object
{
	// TODO
}

public interface Loadable : Object
{
	/**
	 * Whether or not the page is currently loading
	 */
	public abstract bool loading { get; protected set; }

	/**
	 * Progress of a loading operation in the range of [0; 1]
	 */
	public abstract double progress { get; protected set; }

	/**
	 * Request to reload the content
	 *
	 * @param bypass_cache Whether to ignore cache
	 */
	public abstract void reload (bool bypass_cache = false);

	/**
	 * Abort the current loading operation if possible. Action may not happen immediately,
	 * listen to the notify event of loading for further details
	 */
	public abstract void abort_loading ();
}

public interface Zoomable : Object
{
	public abstract double zoom_level { get; set; }

	public abstract void zoom_in ();
	public abstract void zoom_out ();
	public abstract bool can_zoom_in ();
	public abstract bool can_zoom_out ();
	public abstract void reset_zoom ();
}

public interface ResourceContainer : Object
{
	/**
	 * Request to download the resource located at uri
	 *
	 * @param uri The uri of the resource to download
	 */
	public signal void download_resource (string uri);
}

