
namespace Midori
{
	public class UrlBar : Gtk.Box
	{
		/**
		 * The user has submitted the currently entered text and requests the
		 * entered string to be processed
		 *
		 * @param text The text entered by the user
		 */
		public signal void submitted (string text);

		/**
		 * The load action button has been clicked, either reload the page or
		 * stop loading now.
		 */
		public signal void load_action ();

		/**
		 * Indicate whether or not the currently shown page is loading
		 */
		public bool page_is_loading {
			set {
				entry.secondary_icon_name = value ?
					"process-stop-symbolic" : "view-refresh-symbolic";
			}
		}

		/**
		 * The uri of the currently shown page
		 */
		public string page_uri {
			set {
				_page_uri = value;

				update_display_text ();
			}
		}
		string _page_uri;

		/**
		 * Indicate whether or not the currently shown page is loaded via
		 * a secure connection
		 */
		public SecurityStatus page_security_status {
			set {
				switch (value) {
					case SecurityStatus.NONE:
					case SecurityStatus.UNSPECIFIED:
						_page_security_icon_name = "text-html-symbolic";
						break;
					case SecurityStatus.UNKNOWN:
						_page_security_icon_name = "channel-insecure-symbolic";
						break;
					case SecurityStatus.TRUSTED:
						_page_security_icon_name = "channel-secure-symbolic";
						break;
				}

				update_primary_icon ();
			}
		}
		string _page_security_icon_name;

		Gtk.Entry entry;
		UrlBarCompletion completion;

		bool ui_update = false;

		public UrlBar ()
		{
			entry = new Gtk.Entry ();
			entry.activate.connect (() => submitted (entry.text));
			entry.icon_release.connect ((position, event) => {
				if (position == Gtk.EntryIconPosition.SECONDARY)
					load_action ();
			});

			entry.changed.connect (() => {
				if (!ui_update)
					completion.search (entry.text);
			});

			completion = new UrlBarCompletion (entry);
			completion.uri_selected.connect ((uri) => {
				submitted (uri);
			});

			entry.primary_icon_name = "edit-find-symbolic";

			pack_start (entry, true);
		}

		public override void grab_focus ()
		{
			entry.grab_focus ();
		}

		void update_primary_icon ()
		{
			entry.primary_icon_name = _page_security_icon_name;
		}

		void update_display_text ()
		{
			ui_update = true;

			if (!entry.has_focus)
				entry.text = _page_uri;

			ui_update = false;
		}
	}
}

